#!/usr/bin/perl
#
#  Brian Albere
#
#  A script to handle the the parsing of external XML parameter files.  The
#  script also stores the names of the parameters (which should match up with
#  the tags within the XML files) and the default values of the parameters.
#
#  NOTE - This module does not create any logs about its actions
#
#  Edit history
#  06/05/12 rdb received from Brian.
#  06/08/12 rdb Added "use strict"; declared package global variables as "our"
#  06/11/12 rdb Updated all hash table values to remove leading and trailing
#           white space; this allows using spaces in the xml files to make them
#           reasonably readable.
#
use warnings;
use strict;

#rdb try: use XML::Simple qw(:strict);
use XML::Simple;

#software parameter names-------------------------------------------------------
our $cluster_machine = "cluster_machine";  #an array of hashes of machine names
                                       #and number of cores
our $cluster_machine_name = "machine_name";
our $cluster_machine_cores = "cores";

our $log_folder = "log_folder";
our $epimpi_log_folder = "epimpi_log_folder";
our $spatialSmooth_log_folder = "spatialSmooth_log_folder";
our $homoIndex_log_folder = "homoIndex_log_folder";
our $figmdp_log_folder = "figmdp_log_folder";
our $dipTest_log_folder = "dipTest_log_folder";
our $finalGenotypeMapping_log_folder = "finalGenotypeMapping_log_folder";

our $commander_log_file = "commander_log_file";
our $epimpi_log_file = "epimpi_log_file";
our $spatialSmooth_log_file = "spatialSmooth_log_file";
our $homoIndex_log_file = "homoIndex_log_file";
our $figmdp_log_file = "figmdp_log_file";
our $dipTest_log_file = "dipTest_log_file";
our $finalGenotypeMapping_log_file = "finalGenotypeMapping_log_file";

our $status_folder = "status_folder";
our $epimpi_status_folder = "epimpi_status_folder";
our $spatialSmooth_status_folder = "spatialSmooth_status_folder";
our $homoIndex_status_folder = "homoIndex_status_folder";
our $figmdp_status_folder = "figmdp_status_folder";
our $dipTest_status_folder = "dipTest_status_folder";
our $finalGenotypeMapping_status_folder = "finalGenotypeMapping_status_folder";

our $epimpi_status_file = "epimpi_status_file";
our $spatialSmooth_status_file = "spatialSmooth_status_file";
our $homoIndex_status_file = "homoIndex_status_file";
our $figmdp_status_file = "figmdp_status_file";
our $dipTest_status_file = "dipTest_status_file";
our $finalGenotypeMapping_status_file = "finalGenotypeMapping_status_file";

our $mpiexec_path = "mpiexec_path";
our $mpi_processes = "mpi_processes";

our $epimpi_executable = "epimpi_executable";
our $spatialSmooth_script = "spatialSmooth_script";
our $homoIndex_script = "homoIndex_script";
our $figmdp_script = "figmdp_script";
our $dipTest_script = "dipTest_script";
our $finalGenotypeMapping_script = "finalGenotypeMapping_script";

our $spatialSmooth_output_dir = "spatialSmooth_output_dir";
our $logEachSSInstanceFiles = "logEachSSInstanceFiles";

our $homoIndex_output_dir = "homoIndex_output_dir";
our $logEachHIInstanceFiles = "logEachHIInstanceFiles";

our $figmdp_output_dir = "figmdp_output_dir";
our $pre_FIGMDP_homoIndex_mapping_F_output_file = 
                                  "pre_FIGMDP_homoIndex_mapping_F_output_file";
our $pre_FIGMDP_homoIndex_mapping_R_output_file = 
                                  "pre_FIGMDP_homoIndex_mapping_R_output_file";
our $post_FIGMDP_homoIndex_mapping_F_output_file = 
                                  "post_FIGMDP_homoIndex_mapping_F_output_file";
our $post_FIGMDP_homoIndex_mapping_R_output_file = 
                                  "post_FIGMDP_homoIndex_mapping_R_output_file";
our $post_FIGMDP_genotype_mapping_output_file = 
                                  "post_FIGMDP_genotype_mapping_output_file";

our $Rexec_path = "Rexec_path";
our $logEachDTInstanceFiles = "logEachDTInstanceFiles";

our $finalGenotypeMapping_output_dir = "finalGenotypeMapping_output_dir";
our $final_genotype_mapping_output_file = "final_genotype_mapping_output_file";
our $final_homoIndex_mapping_F_output_file = "final_homoIndex_mapping_F_output_file";
our $final_homoIndex_mapping_R_output_file = "final_homoIndex_mapping_R_output_file";
our $final_homoIndex_quality_mapping_output_file = 
                                "final_homoIndex_quality_mapping_output_file";

our $mpi_machine_file = "mpi_machine_file";
our $bio_report_file = "bio_report_file";
our $performance_report_file = "performance_report_file";




#software parameter defaults----------------------------------------------------
our $cluster_machine_default =
            [{$cluster_machine_name => "c0", $cluster_machine_cores => 12},
             {$cluster_machine_name => "c1", $cluster_machine_cores => 8},
             {$cluster_machine_name => "c2", $cluster_machine_cores => 8},
             {$cluster_machine_name => "c3", $cluster_machine_cores => 8}];

our $log_folder_default = "./logFolders";
our $epimpi_log_folder_default = "EPIMPI";
our $spatialSmooth_log_folder_default = "SpatialSmooth";
our $homoIndex_log_folder_default = "HomoIndex";
our $figmdp_log_folder_default = "FIGMDP";
our $dipTest_log_folder_default = "DipTest";
our $finalGenotypeMapping_log_folder_default = "FinalGenotypeMapping";

our $commander_log_file_default = "./Commander.log";
our $epimpi_log_file_default = "EPIMPI";
our $spatialSmooth_log_file_default = "SS";
our $homoIndex_log_file_default = "HI";
our $figmdp_log_file_default = "FIGMDP";
our $dipTest_log_file_default = "DIP";
our $finalGenotypeMapping_log_file_default = "FGM";

our $status_folder_default = "./statusFolders";
our $epimpi_status_folder_default = "EPIMPI";
our $spatialSmooth_status_folder_default = "SpatialSmooth";
our $homoIndex_status_folder_default = "HomoIndex";
our $figmdp_status_folder_default = "FIGMDP";
our $dipTest_status_folder_default = "DipTest";
our $finalGenotypeMapping_status_folder_default = "FinalGenotypeMapping";

our $epimpi_status_file_default = "EPIMPI";
our $spatialSmooth_status_file_default = "SS";
our $homoIndex_status_file_default = "HI";
our $figmdp_status_file_default = "FIGMDP";
our $dipTest_status_file_default = "DIP";
our $finalGenotypeMapping_status_file_default = "FGM";

our $mpiexec_path_default = "~pjh/mpich2/bin/mpiexec";
our $mpi_processes_default = 36;

our $epimpi_executable_default = "./EPIMPI";
our $spatialSmooth_script_default = "./SpatialSmooth.pl";
our $homoIndex_script_default = "./HomoIndex.pl";
our $figmdp_script_default = "./FIGMDP.pl";
our $dipTest_script_default = "./DipTest.pl";
our $finalGenotypeMapping_script_default = "./FinalGenotypeMapping.pl";

our $spatialSmooth_output_dir_default = "./SpatialSmooth_output";
our $logEachSSInstanceFiles_default = 0;

our $homoIndex_output_dir_default = "./HomoIndex_output";
our $logEachHIInstanceFiles_default = 0;

our $figmdp_output_dir_default = "./FIGMDP_output";
our $pre_FIGMDP_homoIndex_mapping_F_output_file_default = "preFIGMDPHomoIndexMapping_F.txt";
our $pre_FIGMDP_homoIndex_mapping_R_output_file_default = "preFIGMDPHomoIndexMapping_R.txt";
our $post_FIGMDP_homoIndex_mapping_F_output_file_default = "postFIGMDPHomoIndexMapping_F.txt";
our $post_FIGMDP_homoIndex_mapping_R_output_file_default = "postFIGMDPHomoIndexMapping_R.txt";
our $post_FIGMDP_genotype_mapping_output_file_default = "postFIGMDPGenotypeMapping.txt";

our $Rexec_path_default = "/usr/bin/R";
our $logEachDTInstanceFiles_default = 0;

our $finalGenotypeMapping_output_dir_default = "./FinalGenotypeMapping_output";
our $final_genotype_mapping_output_file_default = "finalGenotypeMapping.txt";
our $final_homoIndex_mapping_F_output_file_default = "finalHomoIndexMapping_F.txt";
our $final_homoIndex_mapping_R_output_file_default = "finalHomoIndexMapping_R.txt";
our $final_homoIndex_quality_mapping_output_file_default = "finalHomoIndexQualityScores.txt";

our $mpi_machine_file_default = "./MPI_Machine_File.txt";
our $bio_report_file_default = "./bio_report.txt";
our $performance_report_file_default = "performance_report.txt";

#user parameter names-----------------------------------------------------------
our $output_dir = "outputDir";    # directory for all output files
our $parent_allele_file = "parent_allele_file";
our $ndf_file = "ndf_file";
our $bases_files_dir = "bases_files_dir";
our $genotypeOption = "genotypeOption";

our $spatialSmooth = "spatialSmooth";
our $x_slide_dimensions = "x_slide_dimensions";
our $y_slide_dimensions = "y_slide_dimensions";
our $num_zones = "num_zones";
our $smooth_num = "smooth_num";

our $homoIndex = "homoIndex";
our $homoIndexQualityScore_preFilter = "homoIndexQualityScore_preFilter";
our $homoIndexQualityScore_cutoff = "homoIndexQualityScore_cutoff";
our $hetero_cutoff = "hetero_cutoff";
our $homo_cutoff = "homo_cutoff";

our $figmdp = "figmdp";
our $F1_filter = "F1_filter";
our $parent_filter = "parent_filter";
our $NA_snps_filter = "NA_snps_filter";
our $NA_individuals_filter = "NA_individuals_filter";
our $duplicates_filter = "duplicates_filter";
our $F1_filter_OR_or_AND = "F1_filter_OR_or_AND";
our $parent_filter_OR_or_AND = "parent_filter_OR_or_AND";
our $NAs_allowed_per_snp = "NAs_allowed_per_snp";
our $NAs_allowed_per_individual = "NAs_allowed_per_individual";
our $output_F1_filter = "output_F1_filter";
our $output_parent_filter = "output_parent_filter";
our $output_NA_snps_filter = "output_NA_snps_filter";
our $output_NA_individuals_filter = "output_NA_individuals_filter";
our $output_duplicates_filter = "output_duplicates_filter";


our $dipTest = "dipTest";

our $pValue_filter = "pValue_filter";
our $individuals_to_ignore_file = "individuals_to_ignore";
our $pValue_filter_OR_or_AND = "pValue_filter_OR_or_AND";
our $significant_pValue = "significant_pValue";
our $output_pValue_filter = "output_pValue_filter";
our $output_ignored_individuals = "output_ignored_individuals";
our $homoIndexQualityScoreChoice = "homoIndexQualityScoreChoice";

#user parameter defaults--------------------------------------------------------
our $output_dir_default = "./";    # directory for all output files
our $parent_allele_file_default = "./parentAllele.txt";
our $ndf_file_default = "./nimbleGen.ndf";
our $bases_files_dir_default = "./basesFiles";
our $genotypeOption_default = 1;

our $spatialSmooth_default = 1;
our $x_slide_dimensions_default = 480;
our $y_slide_dimensions_default = 640;
our $num_zones_default = 256;
our $smooth_num_default = 100;

our $homoIndex_default = 1;
our $homoIndexQualityScore_preFilter_default = 0;
our $homoIndexQualityScore_cutoff_default = 0.2375;
our $hetero_cutoff_default = 0.525;
our $homo_cutoff_default = 0.475;

our $figmdp_default = 1;
our $F1_filter_default = 1;
our $parent_filter_default = 0;
our $NA_snps_filter_default = 1;
our $NA_individuals_filter_default = 1;
our $duplicates_filter_default = 1;
our $F1_filter_OR_or_AND_default = 1;
our $parent_filter_OR_or_AND_default = 1;
our $NAs_allowed_per_snp_default = 0.25;
our $NAs_allowed_per_individual_default = 0.25;
our $output_F1_filter_default = 0;
our $output_parent_filter_default = 0;
our $output_NA_snps_filter_default = 0;
our $output_NA_individuals_filter_default = 0;
our $output_duplicates_filter_default = 0;

our $dipTest_default = 1;

our $pValue_filter_default = 1;
our $individuals_to_ignore_file_default = "individuals_to_ignore.txt";
our $pValue_filter_OR_or_AND_default = 1;
our $significant_pValue_default = 0.2;
our $output_pValue_filter_default = 0;
our $output_ignored_individuals_default = 0;
our $homoIndexQualityScoreChoice_default = 3;

#variables----------------------------------------------------------------------
my $true = 1;
my $false = 0;

my $xmlSimple = new XML::Simple;
my $softwareParameters;
my $userParameters;

#-------------------------------------------------------------------------------
#
#  This should be called before any other functions are used within this file.
#  It reads in the data from two XML files, the file names should be passed
#  as parameters to the function.  Returns true if the function successfully
#  reads in the XML data and is ready to take requests for parameter values,
#  returns false if unsuccessful
#
sub openParameterFiles
{
    #right number of parameters passed?
    if ( scalar @_ != 2 )
    {
        return $false;
    }
    
    my ($softwareParameters_file, $userParameters_file) = @_;

    #do the parameter files exist?
    if( !open(TEMP, $softwareParameters_file) )
    {
        return $false;
    }
    else
    {
        close(TEMP);
    }
    
    if(!open(TEMP, $userParameters_file))
    {
        return $false;
    }
    else
    {
        close(TEMP);
    }

    #everything is good, open up and parse the files
    $softwareParameters = $xmlSimple -> XMLin( $softwareParameters_file );
    $userParameters = $xmlSimple -> XMLin( $userParameters_file );
    
    #--- rdb added code below; it removes all leading and trailing white
    #    space in all parameters so that the files can be formatted in
    #    a readable way.
    
    while ( my ( $key, $val ) = each( %$userParameters ))
    {
       if ( defined( $val ))
       {
          $val =~ s/^[\s\n]+//;   # remove leading white space
          $val =~ s/[\s\n]+$//;   # remove trailing white space
          $userParameters->{ $key } = $val;
       }
    }
    while ( my ( $key, $val ) = each( %$softwareParameters ))
    {
       if ( defined( $val ))
       {
          $val =~ s/^[\s\n]+//;   # remove leading white space
          $val =~ s/[\s\n]+$//;   # remove trailing white space
          $softwareParameters->{ $key } = $val;
       }
    }
    return $true;
}

#-------------------------------------------------------------------------------
#
#  Use this function to access a user XML parameter, pass in the parameter name
#  and the default value (just in case the parameter was not defined in the XML
#  file).  WARNING - if two parameters are not passed to the function the
#  behavior is unknown
#
sub uParameter
{
    my ($parameter_name, $default_value) = @_;

    if(defined $userParameters -> {$parameter_name})
    {
        return $userParameters -> {$parameter_name};
    }
    else
    {
        #this print can only really be seen by
        #the user and is not logged, could be
        #problematic, but no good current solution TODO
        print STDERR "USER DEFAULT PARAMETER USED: ";
        print STDERR "PARAMETER NAME: $parameter_name : ";
        print STDERR "PARAMETER DEFAULT: $default_value\n";
        return $default_value;
    }
}
#-------------------------------------------------------------------------------
#
#  Accesses xml user parameters for files and directories, but returns
#     the file or directory prefixed with a parent directory
#
sub uDirParameter
{
    my ( $dir, $parameter_name, $default_value) = @_;
    my $parm = &uParameter( $parameter_name, $default_value );
    return "$dir/$parm";
}
#-------------------------------------------------------------------------------
#
#  Use this function to access a software XML parameter, pass in the parameter
#  name and the default value (just in case the parameter was not defined in the
#  XML file).  WARNING - if two parameters are not passed to the function the
#  behavior is unknown
#
sub sParameter
{
    my ($parameter_name, $default_value) = @_;

    if ( defined $softwareParameters -> {$parameter_name})
    {
        return $softwareParameters -> {$parameter_name};
    }
    else
    {
        #this print can only really be seen by
        #the user and is not logged, could be
        #problematic, but no good current solution TODO
        print STDERR "SOFTWARE DEFAULT PARAMETER USED: ";
        print STDERR "PARAMETER NAME: $parameter_name : ";
        print STDERR "PARAMETER DEFAULT: $default_value\n";
        return $default_value;
    }
}
#-------------------------------------------------------------------------------
#
#  Accesses xml user parameters for files and directories, but returns
#     the file or directory prefixed with a parent directory
#
sub sDirParameter
{
    my ( $dir, $parameter_name, $default_value) = @_;
    my $parm = &sParameter( $parameter_name, $default_value );
    return "$dir/$parm";
}
#
#need to end in true in order for Commander.pl to be able to use it
1;
