#!/usr/bin/perl
# 
#          makeTestData.pl - generate a reduced test data set from an NDF
#                 file and a snp file
#
# rdb 06/12/12
#
use strict;
use warnings;

my $usage = q(
      Usage: makeTestData.pl n snpFile ndfFile [outPrefix]
            where n        is # of snps desired
                  snpFile  is existing snpFile
                  ndfFile  is associated ndfFile
                  outPrefix is used for outputfiles; default is "test-n",
                            where n is # snps                  
            OutputFiles
                  outPrefix.snp - reduced set of snps
                  outPrefix.ndf - associated ndf information
);

#-------------------------
my @snpData;
my %snpHash;

#-------------------------

if ( @ARGV < 3 )
{
   print $usage;
   exit;
}

my $snpCount  = $ARGV[ 0 ];
my $snpFileIn = $ARGV[ 1 ];
my $ndfFileIn = $ARGV[ 2 ];
my $prefix = $ARGV[ 3 ];

if ( !defined( $prefix ))
{
   $prefix  = "test-$snpCount";
}                  

open ( SNPIN, "$snpFileIn" ) or die "Unable to open $snpFileIn";
open ( NDFIN, "$ndfFileIn" ) or die "Unable to open $ndfFileIn";
open ( SNPOUT, ">$prefix.snp" ) or die "Unable to open $prefix.snp";
open ( NDFOUT, ">$prefix.ndf" ) or die "Unable to open $prefix.ndf";

# slurp up entire snp file at once
@snpData = <SNPIN>;

# could randomly pick snps, but we'll just take the 1st n for now
for ( my $i = 0; $i < $snpCount; $i++ )
{
   my @fields = split( '\t', $snpData[ $i ] );
   my $snpLoc = "$fields[0]_$fields[1]"; # define location as name_pos
   $snpHash{ $snpLoc } = 1; # just need a defined entry for this position
   print SNPOUT $snpData[ $i ];   # copy to output
}

# Now write the reduced ndf file
while ( my $ndfLine = <NDFIN> )
{
   my @fields = split( "\t", $ndfLine );
   my $loc = $fields[ 4 ];
   if ( defined( $snpHash{ $loc } ))
   {
      print NDFOUT $ndfLine;
   }
}
